@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.235 2006/07/06 18:51:22 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.239 2006/07/20 20:35:46 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -472,6 +472,10 @@ bacon_video_widget_finalize (GObject *object)
 		gdk_cursor_unref (bvw->priv->cursor);
 		bvw->priv->cursor = NULL;
 	}
+	if (bvw->priv->logo_pixbuf != NULL) {
+		gdk_pixbuf_unref (bvw->priv->logo_pixbuf);
+		bvw->priv->logo_pixbuf = NULL;
+	}
 	g_free (bvw->priv->vis_name);
 	g_free (bvw->priv->mediadev);
 	g_object_unref (G_OBJECT (bvw->priv->gc));
@@ -972,6 +976,21 @@ bacon_video_widget_dvd_event (BaconVideoWidget *bvw, BaconVideoWidgetDVDEvent ty
         case BVW_DVD_PREV_ANGLE:
                 event.type = XINE_EVENT_INPUT_ANGLE_PREVIOUS;
                 break;
+        case BVW_DVD_ROOT_MENU_UP:
+                event.type = XINE_EVENT_INPUT_UP;
+                break;
+        case BVW_DVD_ROOT_MENU_DOWN:
+                event.type = XINE_EVENT_INPUT_DOWN;
+                break;
+        case BVW_DVD_ROOT_MENU_LEFT:
+                event.type = XINE_EVENT_INPUT_LEFT;
+                break;
+        case BVW_DVD_ROOT_MENU_RIGHT:
+                event.type = XINE_EVENT_INPUT_RIGHT;
+                break;
+        case BVW_DVD_ROOT_MENU_SELECT:
+                event.type = XINE_EVENT_INPUT_SELECT;
+                break;
         default:
                 return;
         }
@@ -1763,7 +1782,7 @@ bacon_video_widget_expose (GtkWidget *widget, GdkEventExpose *event)
 	/* if there's only audio and no visualisation, draw the logo as well */
 	has_video = xine_get_stream_info(bvw->priv->stream,
 			XINE_STREAM_INFO_HAS_VIDEO);
-	draw_logo = !has_video && !bvw->priv->show_vfx;
+	draw_logo = !has_video && !bvw->priv->using_vfx;
 
 	if (bvw->priv->logo_mode == FALSE && draw_logo == FALSE) {
 		XExposeEvent *expose;
@@ -2468,10 +2487,12 @@ bacon_video_widget_set_logo_mode (BaconVideoWidget *bvw, gboolean logo_mode)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	bvw->priv->logo_mode = logo_mode;
+	if (logo_mode != bvw->priv->logo_mode) {
+		bvw->priv->logo_mode = logo_mode;
 
-	/* Queue a redraw of the widget */
-	gtk_widget_queue_draw (GTK_WIDGET (bvw));
+		/* Queue a redraw of the widget */
+		gtk_widget_queue_draw (GTK_WIDGET (bvw));
+	}
 }
 
 void
@@ -3230,6 +3251,9 @@ bacon_video_widget_set_scale_ratio (BaconVideoWidget *bvw, gfloat ratio)
 	new_h = win_h - widget->allocation.height +
 		bvw->priv->video_height * ratio;
 
+	if (new_w == win_w && new_h == win_h)
+		return;
+
 	/* Change the minimum size of the widget
 	 * but only if we're getting a smaller window */
 	if (new_w < widget->allocation.width
@@ -3525,12 +3549,14 @@ bacon_video_widget_get_metadata_bool (BaconVideoWidget *bvw,
 	switch (type)
 	{
 	case BVW_INFO_HAS_VIDEO:
-		boolean = xine_get_stream_info (bvw->priv->stream,
-				XINE_STREAM_INFO_HAS_VIDEO);
+		if (bvw->priv->logo_mode == FALSE)
+			boolean = xine_get_stream_info (bvw->priv->stream,
+					XINE_STREAM_INFO_HAS_VIDEO);
 		break;
 	case BVW_INFO_HAS_AUDIO:
-		boolean = xine_get_stream_info (bvw->priv->stream,
-				XINE_STREAM_INFO_HAS_AUDIO);
+		if (bvw->priv->logo_mode == FALSE)
+			boolean = xine_get_stream_info (bvw->priv->stream,
+					XINE_STREAM_INFO_HAS_AUDIO);
 		break;
 	default:
 		g_assert_not_reached ();
