@@ -1,7 +1,7 @@
 /* Totem Mozilla plugin
  * 
- * Copyright (C) <2004> Bastien Nocera <hadess@hadess.net>
- * Copyright (C) <2002> David A. Schleef <ds@schleef.org>
+ * Copyright (C) 2004-2006 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2002 David A. Schleef <ds@schleef.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -46,9 +46,11 @@
 #include <nsIDOMWindow.h>
 #include <nsIURI.h>
 #include <nsEmbedString.h>
+#include <nsIInterfaceRequestor.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <docshell/nsIWebNavigation.h>
 
+#define TOTEM_DEBUG 1
 /* define TOTEM_DEBUG for more debug spew */
 #include "debug.h"
 
@@ -65,9 +67,6 @@ cb_update_name (DBusGProxy *proxy, char *svc, char *old_owner,
 	}
 }
 
-/* You don't update, you die! */
-#define MAX_ARGV_LEN 16
-
 static gboolean
 is_supported_scheme (const char *url)
 {
@@ -85,67 +84,90 @@ totem_plugin_fork (TotemPlugin *plugin)
 {
 	GTimeVal then, now;
 	char *svcname;
+	GPtrArray *arr;
 	char **argv;
-	int argc = 0;
 	GError *err = NULL;
 	totemMozillaObject *iface = plugin->iface;
 
-	argv = (char **)g_new0 (char *, MAX_ARGV_LEN);
+	/* Make sure we don't get both an XID and hidden */
+	if (plugin->window && plugin->hidden) {
+		g_warning ("Both hidden and a window!");
+		return FALSE;
+	}
+
+	arr = g_ptr_array_new ();
 
 	if (g_file_test ("./totem-mozilla-viewer",
 				G_FILE_TEST_EXISTS) != FALSE) {
-		argv[argc++] = g_strdup ("./totem-mozilla-viewer");
+		g_ptr_array_add (arr, g_strdup ("./totem-mozilla-viewer"));
 	} else {
-		argv[argc++] = g_strdup (LIBEXECDIR"/totem-mozilla-viewer");
+		g_ptr_array_add (arr,
+				g_strdup (LIBEXECDIR"/totem-mozilla-viewer"));
 	}
 
-	argv[argc++] = g_strdup (TOTEM_OPTION_XID);
-	argv[argc++] = g_strdup_printf ("%lu", plugin->window);
+	if (plugin->window) {
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_XID));
+		g_ptr_array_add (arr, g_strdup_printf ("%lu", plugin->window));
+	}
 
 	if (plugin->width) {
-		argv[argc++] = g_strdup (TOTEM_OPTION_WIDTH);
-		argv[argc++] = g_strdup_printf ("%d", plugin->width);
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_WIDTH));
+		g_ptr_array_add (arr, g_strdup_printf ("%d", plugin->width));
 	}
 
 	if (plugin->height) {
-		argv[argc++] = g_strdup (TOTEM_OPTION_HEIGHT);
-		argv[argc++] = g_strdup_printf ("%d", plugin->height);
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HEIGHT));
+		g_ptr_array_add (arr, g_strdup_printf ("%d", plugin->height));
 	}
 
 	if (plugin->src) {
-		argv[argc++] = g_strdup (TOTEM_OPTION_URL);
-		argv[argc++] = g_strdup (plugin->src);
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_URL));
+		g_ptr_array_add (arr, g_strdup (plugin->src));
 	}
-	
+
 	if (plugin->href) {
-		argv[argc++] = g_strdup (TOTEM_OPTION_HREF);
-		argv[argc++] = g_strdup (plugin->href);
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HREF));
+		g_ptr_array_add (arr, g_strdup (plugin->href));
 	}
 
 	if (plugin->target) {
-		argv[argc++] = g_strdup (TOTEM_OPTION_TARGET);
-		argv[argc++] = g_strdup (plugin->target);
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_TARGET));
+		g_ptr_array_add (arr, g_strdup (plugin->target));
+	}
+
+	if (plugin->mimetype) {
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_MIMETYPE));
+		g_ptr_array_add (arr, g_strdup (plugin->mimetype));
 	}
 
 	if (plugin->controller_hidden) {
-		argv[argc++] = g_strdup ("--nocontrols");
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_CONTROLS_HIDDEN));
+	}
+
+	if (plugin->hidden) {
+		g_ptr_array_add (arr, g_strdup (TOTEM_OPTION_HIDDEN));
 	}
 
 	if (is_supported_scheme (plugin->src) == FALSE)
-		argv[argc++] = g_strdup (plugin->src);
+		g_ptr_array_add (arr, g_strdup (plugin->src));
 	else
-		argv[argc++] = g_strdup ("fd://0");
+		g_ptr_array_add (arr, g_strdup ("fd://0"));
 
-	argv[argc] = NULL;
+	g_ptr_array_add (arr, NULL);
+	argv = (char **) g_ptr_array_free (arr, FALSE);
 
 #ifdef TOTEM_DEBUG
 	{
+		GString *s;
 		int i;
-		g_print ("Launching: ");
-		for (i = 0; i < argc; i++) {
-			g_print ("%s ", argv[i]);
+
+		s = g_string_new ("Launching: ");
+		for (i = 0; argv[i] != NULL; i++) {
+			g_string_append (s, argv[i]);
 		}
-		g_print ("\n");
+		g_string_append (s, "\n");
+		D("%s", s->str);
+		g_string_free (s, TRUE);
 	}
 #endif
 
@@ -156,9 +178,8 @@ totem_plugin_fork (TotemPlugin *plugin)
 	{
 		D("Spawn failed");
 
-		if (err)
-		{
-			fprintf(stderr, "%s\n", err->message);
+		if (err) {
+			g_warning ("%s\n", err->message);
 			g_error_free(err);
 		}
 
@@ -168,11 +189,12 @@ totem_plugin_fork (TotemPlugin *plugin)
 
 	g_strfreev (argv);
 
+	fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
+
 	/* now wait until startup is complete */
 	plugin->got_svc = FALSE;
-	plugin->wait_for_svc =
-		g_strdup_printf ("org.totem_%d.MozillaPluginService",
-				 plugin->player_pid);
+	plugin->wait_for_svc = g_strdup_printf
+		("org.totem_%d.MozillaPluginService", plugin->player_pid);
 	D("waiting for signal %s", plugin->wait_for_svc);
 	dbus_g_proxy_add_signal (plugin->proxy, "NameOwnerChanged",
 				 G_TYPE_STRING, G_TYPE_STRING,
@@ -188,6 +210,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 		g_get_current_time (&now);
 	} while (iface->tm != NULL && !plugin->got_svc &&
 		 (now.tv_sec <= then.tv_sec));
+
 	if (!iface->tm) {
 		/* we were destroyed in one of the iterations of the
 		 * mainloop, get out ASAP */
@@ -199,7 +222,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 	dbus_g_proxy_disconnect_signal (plugin->proxy, "NameOwnerChanged",
 					G_CALLBACK (cb_update_name), plugin);
 	if (!plugin->got_svc) {
-		fprintf (stderr, "Failed to receive DBUS interface response\n");
+		g_warning ("Failed to receive DBUS interface response\n");
 		g_free (plugin->wait_for_svc);
 
 		if (plugin->player_pid) {
@@ -237,7 +260,7 @@ resolve_relative_uri (nsIURI *docURI, const char *uri)
 	return g_strdup (uri);
 }
 
-static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
+static NPError totem_plugin_new_instance (NPMIMEType mimetype, NPP instance,
 		uint16_t mode, int16_t argc, char *argn[], char *argv[],
 		NPSavedData *saved)
 {
@@ -287,8 +310,9 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 	}
 
 	/* mode is NP_EMBED, NP_FULL, or NP_BACKGROUND (see npapi.h) */
+	//FIXME we should error out if we are in fullscreen mode
 	printf("mode %d\n",mode);
-	//printf("mime type: %s\n",pluginType);
+	printf("mime type: %s\n", mimetype);
 	plugin->instance = instance;
 	plugin->send_fd = -1;
 
@@ -296,19 +320,32 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 	nsIDOMWindow *domWin = nsnull;
 	mozilla_functions.getvalue (instance, NPNVDOMWindow,
 				    NS_REINTERPRET_CAST (void**, &domWin));
-	nsIWebNavigation *webNav = nsnull;
+	nsIInterfaceRequestor *irq = nsnull;
 	if (domWin) {
-		domWin->QueryInterface (NS_GET_IID (nsIWebNavigation),
-					NS_REINTERPRET_CAST (void**, &webNav));
+		domWin->QueryInterface (NS_GET_IID (nsIInterfaceRequestor),
+					NS_REINTERPRET_CAST (void**, &irq));
 		NS_RELEASE (domWin);
 	}
 
+	nsIWebNavigation *webNav = nsnull;
+	if (irq) {
+		irq->GetInterface (NS_GET_IID (nsIWebNavigation),
+				   NS_REINTERPRET_CAST (void**, &webNav));
+		NS_RELEASE (irq);
+	}
+
 	nsIURI *docURI = nsnull;
 	if (webNav) {
 		webNav->GetCurrentURI (&docURI);
 		NS_RELEASE (webNav);
 	}
 
+	/* Set the default cache behaviour */
+	if (strcmp (mimetype, "video/quicktime") != 0) {
+		plugin->cache = TRUE;
+	}
+	plugin->mimetype = g_strdup (mimetype);
+
 	for (i=0; i<argc; i++) {
 		printf ("argv[%d] %s %s\n", i, argn[i], argv[i]);
 		if (g_ascii_strcasecmp (argn[i],"width") == 0) {
@@ -323,17 +360,32 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 		if (g_ascii_strcasecmp (argn[i], "href") == 0) {
 			plugin->href = resolve_relative_uri (docURI, argv[i]);
 		}
+		if (g_ascii_strcasecmp (argn[i], "cache") == 0) {
+			plugin->cache = TRUE;
+			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
+				plugin->cache = FALSE;
+			}
+		}
 		if (g_ascii_strcasecmp (argn[i], "target") == 0) {
 			plugin->target = g_strdup (argv[i]);
 		}
+		/* Quicktime parameter */
 		if (g_ascii_strcasecmp (argn[i], "controller") == 0) {
 			if (g_ascii_strcasecmp (argv[i], "false") == 0) {
 				plugin->controller_hidden = TRUE;
 			}
 			//FIXME see http://www.htmlcodetutorial.com/embeddedobjects/_EMBED_CONTROLS.html
 		}
+		/* Windows Media Player parameter */
+		if (g_ascii_strcasecmp (argn[i], "uimode") == 0) {
+			if (g_ascii_strcasecmp (argv[i], "none") == 0) {
+				plugin->controller_hidden = TRUE;
+			}
+		}
 		if (g_ascii_strcasecmp (argn[i], "hidden") == 0) {
-			//FIXME
+			if (g_ascii_strcasecmp (argv[i], "false") != 0) {
+				plugin->hidden = TRUE;
+			}
 		}
 		if (g_ascii_strcasecmp (argn[i], "autostart") == 0
 				|| g_ascii_strcasecmp (argn[i], "autoplay") == 0) {
@@ -372,7 +424,6 @@ static NPError totem_plugin_destroy_instance (NPP instance, NPSavedData **save)
 	if (!plugin || !plugin->iface || !plugin->iface->tm)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
-	plugin->iface->Rewind ();
 	plugin->iface->invalidatePlugin ();
 
 	if (plugin->send_fd >= 0)
@@ -428,9 +479,6 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 		plugin->window = (Window) window->window;
 		if (!totem_plugin_fork (plugin))
 			return NPERR_GENERIC_ERROR;
-
-		if (plugin->send_fd > 0)
-			fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
 	}
 
 	D("leaving plugin_set_window");
@@ -441,11 +489,27 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 static NPError totem_plugin_new_stream (NPP instance, NPMIMEType type,
 		NPStream* stream_ptr, NPBool seekable, uint16* stype)
 {
+	TotemPlugin *plugin;
+
 	D("plugin_new_stream");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
+	plugin = (TotemPlugin *) instance->pdata;
+
+	D("plugin_new_stream type: %s url: %s", type, plugin->src);
+
+	//FIXME need to find better semantics?
+	//what about saving the state, do we get confused?
+	if (g_str_has_prefix (plugin->src, "file://")) {
+		*stype = NP_ASFILEONLY;
+		plugin->stream_type = NP_ASFILEONLY;
+	} else {
+		*stype = NP_ASFILE;
+		plugin->stream_type = NP_ASFILE;
+	}
+
 	return NPERR_NO_ERROR;
 }
 
@@ -486,20 +550,13 @@ static int32 totem_plugin_write_ready (NPP instance, NPStream *stream)
 	if (is_supported_scheme (plugin->src) == FALSE)
 		return 0;
 
-	if (plugin->send_fd < 0) {
-		if (!totem_plugin_fork (plugin))
-			return 0;
-
-		if (plugin->send_fd > 0)
-			fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
-		else
-			return 0;
-	}
+	if (plugin->send_fd < 0)
+		return (8 * 1024);
 
 	fds.events = POLLOUT;
 	fds.fd = plugin->send_fd;
 	if (plugin->send_fd > 0 && poll (&fds, 1, 0) > 0)
-		return (8*1024);
+		return (8 * 1024);
 
 	return 0;
 }
@@ -520,8 +577,13 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 	if (plugin == NULL)
 		return -1;
 
-	if (!plugin->player_pid)
-		return -1;
+	if (!plugin->player_pid) {
+		//FIXME push the data to the playlist parser to get
+		//something recognised first?
+
+		if (!totem_plugin_fork (plugin))
+			return -1;
+	}
 
 	if (plugin->send_fd < 0)
 		return -1;
@@ -541,8 +603,9 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 	const char* fname)
 {
 	TotemPlugin *plugin;
+	GError *err = NULL;
 
-	D("plugin_stream_as_file");
+	D("plugin_stream_as_file: %s", fname);
 
 	if (instance == NULL)
 		return;
@@ -551,6 +614,12 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 	if (plugin == NULL)
 		return;
 
+	if (!dbus_g_proxy_call (plugin->proxy, "SetLocalFile", &err,
+			G_TYPE_STRING, fname, G_TYPE_INVALID,
+			G_TYPE_INVALID)) {
+		g_warning ("Error: %s\n", err->message);
+	}
+
 	D("plugin_stream_as_file\n");
 }
 
@@ -574,17 +643,18 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 	TotemPlugin *plugin;
 	NPError err = NPERR_NO_ERROR;
 
+	/* See NPPVariable in npapi.h */
 	D("plugin_get_value %d\n", variable);
 
 	switch (variable) {
 	case NPPVpluginNameString:
-		*((char **)value) = "Totem Mozilla Plugin";
+		*((char **)value) = "Totem Web Browser Plugin";
 		break;
 	case NPPVpluginDescriptionString:
 		*((char **)value) = totem_plugin_get_description();
 		break;
 	case NPPVpluginNeedsXEmbed:
-		*((PRBool *)value) = PR_TRUE;
+		*((NPBool *)value) = PR_TRUE;
 		break;
 	case NPPVpluginScriptableIID: {
 		static nsIID sIID = TOTEMMOZILLASCRIPT_IID;
@@ -638,10 +708,10 @@ NP_GetValue(void *future, NPPVariable variable, void *value)
 }
 
 static struct {
-	const char *mime_type;
+	const char *mimetype;
 	const char *extensions;
 	const char *mime_alias;
-} mime_types[] = {
+} mimetypes[] = {
 	{ "video/quicktime", "mov", NULL },
 	{ "application/x-mplayer2", "avi, wma, wmv", "video/x-msvideo" },
 	{ "video/mpeg", "mpg, mpeg, mpe", NULL },
@@ -652,7 +722,7 @@ static struct {
 	{ "video/divx", "divx", "video/x-msvideo" },
 	{ "audio/wav", "wav", NULL }
 };
-#define NUM_MIME_TYPES G_N_ELEMENTS(mime_types)
+#define NUM_MIME_TYPES G_N_ELEMENTS(mimetypes)
 
 char *NP_GetMIMEDescription(void)
 {
@@ -668,17 +738,17 @@ char *NP_GetMIMEDescription(void)
 		const char *desc;
 		char *item;
 
-		desc = gnome_vfs_mime_get_description (mime_types[i].mime_type);
-		if (desc == NULL && mime_types[i].mime_alias != NULL) {
+		desc = gnome_vfs_mime_get_description (mimetypes[i].mimetype);
+		if (desc == NULL && mimetypes[i].mime_alias != NULL) {
 			desc = gnome_vfs_mime_get_description
-				(mime_types[i].mime_alias);
+				(mimetypes[i].mime_alias);
 		}
 		if (desc == NULL) {
-			desc = mime_types[i].mime_alias;
+			desc = mimetypes[i].mime_alias;
 		}
 
-		item = g_strdup_printf ("%s:%s:%s;", mime_types[i].mime_type,
-				mime_types[i].extensions, desc);
+		item = g_strdup_printf ("%s:%s:%s;", mimetypes[i].mimetype,
+				mimetypes[i].extensions, desc);
 		list = g_string_append (list, item);
 		g_free (item);
 	}
@@ -692,7 +762,7 @@ NPError NP_Initialize (NPNetscapeFuncs * moz_funcs,
 		NPPluginFuncs * plugin_funcs)
 {
 	NPError err = NPERR_NO_ERROR;
-	PRBool supportsXEmbed = PR_FALSE;
+	NPBool supportsXEmbed = PR_FALSE;
 	NPNToolkitType toolkit = (NPNToolkitType) 0;
 
 	D("NP_Initialize\n");
