@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * Copyright (C) 2001-2005 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2001-2006 Bastien Nocera <hadess@hadess.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -758,19 +758,15 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		/* Set the logo */
 		bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
-		bacon_video_widget_set_logo (totem->bvw, LOGO_PATH);
-
+		gtk_widget_set_sensitive (GTK_WIDGET (totem->properties),
+				FALSE);
 		update_mrl_label (totem, NULL);
 	} else {
 		gboolean caps;
 		char *subtitle_uri;
 		GError *err = NULL;
 
-		if (bacon_video_widget_get_logo_mode (totem->bvw) != FALSE)
-		{
-			bacon_video_widget_close (totem->bvw);
-			bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
-		}
+		bacon_video_widget_set_logo_mode (totem->bvw, FALSE);
 
 		subtitle_uri = totem_uri_get_subtitle_uri (mrl);
 		totem_gdk_window_set_waiting_cursor (totem->win->window);
@@ -801,6 +797,8 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		/* Take a screenshot */
 		totem_main_set_sensitivity ("tmw_take_screenshot_menu_item",
 				retval);
+		gtk_widget_set_sensitive
+			(GTK_WIDGET (totem->properties), retval);
 
 		/* Set the playlist */
 		totem_playlist_set_playing (totem->playlist, retval);
@@ -829,13 +827,12 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 			g_free (totem->mrl);
 			totem->mrl = NULL;
 			play_pause_set_label (totem, STATE_STOPPED);
+			bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
 		}
 	}
 	update_buttons (totem);
 	update_media_menu_items (totem);
 
-	totem_main_set_sensitivity ("tmw_properties_menu_item", retval);
-
 	return retval;
 }
 
@@ -2031,46 +2028,6 @@ on_take_screenshot1_activate (GtkButton *button, Totem *totem)
 	gdk_pixbuf_unref (pixbuf);
 }
 
-static void
-hide_props_dialog (GtkWidget *widget, int trash, gpointer user_data)
-{
-	gtk_widget_hide (widget);
-}
-
-static void
-on_properties1_activate (GtkButton *button, Totem *totem)
-{
-	static GtkWidget *dialog;
-
-	if (totem->properties == NULL)
-	{
-		totem_action_error (_("Totem couldn't show the movie properties window."), _("Make sure that Totem is correctly installed."), totem);
-		return;
-	}
-
-	if (dialog != NULL)
-	{
-		gtk_widget_show (dialog);
-		return;
-	}
-
-	dialog = gtk_dialog_new_with_buttons (_("Properties"),
-			GTK_WINDOW (totem->win),
-			GTK_DIALOG_DESTROY_WITH_PARENT
-			| GTK_DIALOG_NO_SEPARATOR,
-			GTK_STOCK_CLOSE,
-			GTK_RESPONSE_ACCEPT,
-			NULL);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-			totem->properties, TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	g_signal_connect (G_OBJECT (dialog), "response",
-			G_CALLBACK (hide_props_dialog), NULL);
-	g_signal_connect (G_OBJECT (dialog), "delete-event",
-			G_CALLBACK (hide_props_dialog), NULL);
-	gtk_widget_show (dialog);
-}
-
 static void
 on_preferences1_activate (GtkButton *button, Totem *totem)
 {
@@ -2295,6 +2252,45 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 			g_free (title);
 		}
 		break;
+	case TOTEM_REMOTE_COMMAND_UP:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU_UP);
+		break;
+	case TOTEM_REMOTE_COMMAND_DOWN:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU_DOWN);
+		break;
+	case TOTEM_REMOTE_COMMAND_LEFT:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU_LEFT);
+		break;
+	case TOTEM_REMOTE_COMMAND_RIGHT:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU_RIGHT);
+		break;
+	case TOTEM_REMOTE_COMMAND_SELECT:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU_SELECT);
+		break;
+	case TOTEM_REMOTE_COMMAND_DVD_MENU:
+		bacon_video_widget_dvd_event (totem->bvw,
+				BVW_DVD_ROOT_MENU);
+		break;
+	case TOTEM_REMOTE_COMMAND_ZOOM_UP:
+		totem_action_zoom_relative (totem, ZOOM_IN_OFFSET);
+		break;
+	case TOTEM_REMOTE_COMMAND_ZOOM_DOWN:
+		totem_action_zoom_relative (totem, ZOOM_OUT_OFFSET);
+		break;
+	case TOTEM_REMOTE_COMMAND_EJECT:
+		totem_action_eject (totem);
+		break;
+	case TOTEM_REMOTE_COMMAND_PLAY_DVD:
+		// TODO - how to see if can, and play the DVD (like the menu item)
+		break;
+	case TOTEM_REMOTE_COMMAND_MUTE:
+		totem_action_volume_relative (totem, -100);
+		break;
 	default:
 		handled = FALSE;
 		break;
@@ -3105,9 +3101,6 @@ totem_callback_connect (Totem *totem)
 	item = glade_xml_get_widget (totem->xml, "tmw_preferences_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_preferences1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_properties_menu_item");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_properties1_activate), totem);
 	item = glade_xml_get_widget (totem->xml, "tmw_volume_up_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_volume_up1_activate), totem);
@@ -3586,6 +3579,7 @@ main (int argc, char **argv)
 
 	/* The sidebar */
 	playlist_widget_setup (totem);
+	totem->properties = bacon_video_widget_properties_new ();
 	totem_sidebar_setup (totem);
 
 	/* The rest of the widgets */
@@ -3617,9 +3611,6 @@ main (int argc, char **argv)
 			"tcw_time_display_label");
 	totem->seek_lock = totem->vol_lock = totem->vol_fs_lock = FALSE;
 
-	/* Properties */
-	totem->properties = bacon_video_widget_properties_new ();
-
 	totem_session_setup (totem, argv);
 	totem_setup_recent (totem);
 	totem_setup_file_monitoring (totem);
@@ -3641,6 +3632,7 @@ main (int argc, char **argv)
 	video_widget_create (totem);
 	long_action ();
 	gtk_widget_grab_focus (GTK_WIDGET (totem->bvw));
+	bacon_video_widget_set_logo (totem->bvw, LOGO_PATH);
 
 	/* The prefs after the video widget is connected */
 	totem_setup_preferences (totem);
