@@ -71,7 +71,7 @@ static const GtkTargetEntry source_table[] = {
 
 static void vanity_action_exit (Vanity *vanity);
 
-static const struct poptOption options[] = {
+static struct poptOption options[] = {
 	{NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, N_("Backend options"), NULL},
 	{"debug", '\0', POPT_ARG_NONE, NULL, 0, N_("Debug mode on"), NULL},
 	{NULL, '\0', 0, NULL, 0} /* end the list */
@@ -258,7 +258,7 @@ on_about1_activate (GtkButton *button, Vanity *vanity)
 
 	vanity->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 			"name", _("Vanity"),
-			"version", VERSION,
+			"version", PACKAGE_VERSION,
 			"copyright", _("Copyright \xc2\xa9 2002-2005 Bastien Nocera"),
 			"comments", description,
 			"authors", authors,
@@ -594,8 +594,8 @@ video_widget_create (Vanity *vanity)
 
 		bacon_video_widget_play (vanity->bvw, &err);
 	} else {
-		bacon_video_widget_set_logo_mode (vanity->bvw, TRUE);
 		bacon_video_widget_set_logo (vanity->bvw, LOGO_PATH);
+		bacon_video_widget_set_logo_mode (vanity->bvw, TRUE);
 		g_message ("%s", LOGO_PATH);
 	}
 
@@ -655,7 +655,7 @@ main (int argc, char **argv)
 
 	gtk_init (&argc, &argv);
 #ifndef HAVE_GTK_ONLY
-	gnome_program_init ("vanity", VERSION,
+	gnome_program_init ("vanity", PACKAGE_VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			GNOME_PARAM_APP_DATADIR, DATADIR,
